/****************************************************************************
 * This file is part of the project AqDatabase.
 * AqDatabase (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_H
#define AQDB_H


#define AQDB_LOGDOMAIN "aqdb"

#include <aqdatabase/aqdb_api.h>

#include <gwenhywfar/i18n.h>

#include <inttypes.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



/** @mainpage AqDatabase Documentation Main Page

 AqDatabase provides a simple API for a table and object based database.

 A AqDatabase contains a list of tables, each table contains objects, each object contains fields.

 A table is a description of an object type. i.e. it contains a description of the fields each object
 of the given type contains.

 AqDatabase allows creating tables, creating, deleting and modifying objects. There is a builtin
 mechanism to rollback changes.

 Currently there are two implementations: DbDir and DbFile.

 DbDir uses a folder with multiple files to store the data.

 DbFile stores all data of a database in a single file and is therefore easier to work with. It is also
 newer and faster than DbDir.

 */




#define AQDB_PM_LIBNAME    "aqdatabase"
#define AQDB_PM_SYSCONFDIR "sysconfdir"
#define AQDB_PM_DATADIR    "datadir"
#define AQDB_PM_LOCALEDIR  "localedir"




typedef enum {
  AQDB_DataType_Unknown=-1,
  AQDB_DataType_None=0,
  AQDB_DataType_Int,
  AQDB_DataType_String,
  AQDB_DataType_Bin,
  AQDB_DataType_Date,
  AQDB_DataType_Timestamp,
  AQDB_DataType_Amount
} AQDB_DATA_TYPE;


typedef enum {
  AQDB_ActionType_Unknown=-1,
  AQDB_ActionType_None=0,
  AQDB_ActionType_Add,
  AQDB_ActionType_Remove,
  AQDB_ActionType_Modify
} AQDB_ACTION_TYPE;


/* can be used with ReadObject and QuerySubmit functions */
#define AQDB_ACCESS_FLAGS_ISWRITER   0x80000000


/** @name Flags
 *
 */
/*@{*/

/** mask for action flags (read, write etc) */
#define AQDB_ACTION_FLAGS_MASK       0x00ff0000

/** deprecated */
#define   AQDB_ACTION_FLAGS_CREATE   0x00010000
/** open database or table for reading */
#define   AQDB_ACTION_FLAGS_READ     0x00020000

/** open database or table for writing */
#define   AQDB_ACTION_FLAGS_WRITE    0x00040000

/** deprecated */
#define   AQDB_ACTION_FLAGS_LOCK     0x00080000

/** used with @ref AQDB_DB_EndEdit to abort an edit session */
#define   AQDB_ACTION_FLAGS_ABORT    0x00100000

/** mask for permission flags (currently not used) */
#define AQDB_PERM_FLAGS_MASK         0xff000000

/** permission to add objects */
#define   AQDB_PERM_FLAGS_ADD        0x01000000

/** permission to delete objects */
#define   AQDB_PERM_FLAGS_DEL        0x02000000

/** permission to modify objects */
#define   AQDB_PERM_FLAGS_EDIT       0x04000000

/** deprecated, don't use */
#define   AQDB_PERM_FLAGS_LOG        0x08000000

/** deprecated, don't use */
#define   AQDB_PERM_FLAGS_SIGN       0x10000000


/*@}*/


typedef uint32_t AQDB_ID;
#define AQDB_ID_BITS           32
#define AQDB_ID_TOPHALFMASK    0xffff0000
#define AQDB_ID_BOTTOMHALFMASK 0x0000ffff


/** @name Query Flags
 *
 */
/*@{*/

/**
 * This flag makes the query code stop on the first matching object.
 */
#define AQDB_QUERY_FLAGS_STOPATFIRST 0x00000001
/*@}*/


#define AQDB_TABLE_NAME_SESSIONTABLE  "_sessionlog"
#define AQDB_TABLE_NAME_ROLLBACKTABLE "_rollbacklog"


#ifdef __cplusplus
extern "C" {
#endif

AQDB_API int AQDB_Init();
AQDB_API int AQDB_Fini();

AQDB_API AQDB_DATA_TYPE AQDB_DataType_fromString(const char *s);
AQDB_API const char *AQDB_DataType_toString(AQDB_DATA_TYPE t);

#ifdef __cplusplus
}
#endif



#endif

